require 'iron_mailer/send_email'

describe IronMailer::SendEmail do
  describe "initializer" do
    before(:each) do
      @options = {
        'iron_mq'=>{'token'=>'some set value', 'project_id'=>'some-24-char-project-id-', 'queue_name'=>'some_test_queue'},
        'iron_cache'=>{'token'=>'some set value', 'project_id'=>'some-24-char-project-id-', 'cache_name'=>'some_test_cache_name'},
        'delivery_method'=>{
          'smtp'=>{
            'address'=>'smtp.example.com',
            'port'=>587,
            'domain'=>'somedomain.com',
            'user_name'=>'username',
            'password'=>'password',
            'authentication'=>'plain',
            'enable_starttls_auto'=>'enable_starttls_auto'
          }
        }
      }
      @send_email = IronMailer::SendEmail.new(@options)
    end

    it "should allow setting of a test delivery_method" do
      IronMailer::SendEmail.new(@options.merge('delivery_method'=>'test'))
      @send_email.delivery_method.should == :smtp
      @send_email.delivery_method_settings.class.should == Hash
    end

    describe "iron_mq_client" do

      it "should assign the token" do
        @send_email.iron_mq_client.token.should == @options['iron_mq']['token']
      end

      it "should assign the project_id" do
        @send_email.iron_mq_client.project_id.should == @options['iron_mq']['project_id']
      end

      it "should assign the queue with the provided queue name" do
        @send_email.queue.name.should == @options['iron_mq']['queue_name']
      end
    end

    describe "iron_cache_client" do
      it "should assign the token" do
        @send_email.iron_cache_client.token.should == @options['iron_cache']['token']
      end

      it "should assign the project_id" do
        @send_email.iron_cache_client.project_id.should == @options['iron_cache']['project_id']
      end

      it "should assign the cache with the provided cache name" do
        @send_email.cache.name.should == @options['iron_cache']['cache_name']
      end
    end

    it "should set delivery_method" do
       @send_email.delivery_method.should == @options['delivery_method'].keys.first.to_sym
    end

    describe "smtp_config" do
      it "should assign the address" do
        @send_email.delivery_method_settings[:address].should == @options['delivery_method']['smtp']['address']
      end

      it "should assign the port" do
        @send_email.delivery_method_settings[:port] == @options['delivery_method']['smtp']['port']
      end

      it "should assign the domain" do
        @send_email.delivery_method_settings[:domain] == @options['delivery_method']['smtp']['domain']
      end

      it "should assign the user_name" do
        @send_email.delivery_method_settings[:user_name] == @options['delivery_method']['smtp']['user_name']
      end

      it "should assign the password" do
        @send_email.delivery_method_settings[:password] == @options['delivery_method']['smtp']['password']
      end

      it "should assign the authentication" do
        @send_email.delivery_method_settings[:authentication] == @options['delivery_method']['smtp']['authentication']
      end

      it "should assign enable_starttls_auto" do
        @send_email.delivery_method_settings[:enable_starttls_auto] == @options['delivery_method']['smtp']['enable_starttls_auto']
      end
    end

  end
  
  describe "execute" do    
    before(:each) do
      @iron_mq_config = JSON.parse(File.read('spec/config/iron_mq.json'))
      @send_email = IronMailer::SendEmail.new(test_config)
      @send_email.queue.clear if @send_email.queue.size > 0
      
      begin
        @send_email.cache.size
      rescue Rest::HttpError => e
        @send_email.cache.put("some_key", 'some_value')
      end
      @test_email = test_email
      @message = @send_email.queue.post(@test_email)

      @send_email.queue.reload
      @send_email.cache.reload
      
      @execute=lambda{
        @send_email.execute
        @send_email.queue.reload
        @send_email.cache.reload
      }      
    end
    
    after(:each) do
      @send_email.queue.clear if @send_email.queue.size > 0
      @send_email.cache.clear
    end

    it "should remove a message from the queue" do
      @execute.should change(@send_email.queue, :size)
    end

    it "should add info about the email to the cache" do
      @send_email.should_receive(:cache_email)
      @execute.call
    end

    it "should call deliver with on the results of split_email" do
      test_email=double("email")
      emails = [test_email]
      @send_email.stub(:split_email).and_return(emails)
    end

    it "should add the message back to the queue if there's an error " do
      @send_email.queue.clear if @send_email.cache.size > 0
      email = test_email
      email.to.clear
      class ErrorMailer
        def initialize(*args)
        end
        def deliver!(*args)
          raise
        end
      end
      Mail.defaults do
        delivery_method ErrorMailer
      end

      @send_email.queue.post(:error)
      @execute.call
      @send_email.queue.get.should_not be_nil
    end

    it "should send 'all' queued emails" do
      @send_email.queue.post(test_email)
      @send_email.queue.post(test_email)
      @execute.should change(@send_email.queue, :size).to(0)
    end
  end

  describe "cache_email" do
    before(:each) do
      @send_email = IronMailer::SendEmail.new(test_config)
      @test_email = @send_email.split_email(test_email).first
      @cache=lambda{
        @send_email.cache_email(@test_email)
        @send_email.cache.reload
      }
    end


    after(:each) do
      @send_email.cache.clear if @send_email.cache.size > 0
    end

    it "should add an item to the cache " do
      @cache.should change(@send_email.cache, :size).by(1)
    end

    it "should use the cache key from the header" do
      @cache.call
      cache_key=JSON.parse(@test_email.header['X-SMTPAPI'].value)['unique_args']['cache_key']
      @send_email.cache.get(cache_key).should_not be_nil
    end

    it "should blow up if the cache key is blank" do
      @test_email.header['X-SMTPAPI'] = nil
      @test_email.header['X-SMTPAPI'] = {unique_args: {cache_key: nil}}.to_json
      @cache.should raise_error
    end

    it "should blow up if the header is blank" do
      @test_email.header['X-SMTPAPI'] = nil
      @test_email.header['X-SMTPAPI'] = ""
      @cache.should raise_error
    end

    it "should handle 'X-SMTPAPI' being an array" do
      @test_email.header['X-SMTPAPI'] = 'some_header_value'
      @cache.should change(@send_email.cache, :size).by(1)
    end

  end
  describe "extract_cache_info" do
    before(:each) do
      @send_email = IronMailer::SendEmail.new(test_config)
      @test_email = test_email(['someother@example.com'])
    end
    it "should extract the subject" do
      @send_email.extract_cache_info(@test_email)['subject'].should == @test_email.subject
    end

    it "should extract the sender" do
      @send_email.extract_cache_info(@test_email)['from'].should == @test_email.from
    end

    it "should extract the recipient" do
      @send_email.extract_cache_info(@test_email)['to'].should == @test_email.to
    end

    it "should extract the date" do
      @test_email.stub(:date).and_return(DateTime.now)
      @send_email.extract_cache_info(@test_email)['date'].should == @test_email.date
    end

    it "should extract the content" do
      @send_email.extract_cache_info(@test_email)['mail'].should == @test_email.to_s
    end
  end

  describe "split_email" do
    before(:each) do
      @send_email = IronMailer::SendEmail.new(test_config)
      @test_email = test_email(['someother@example.com'])
    end

    it "should split a single email into one per recipient" do
      @send_email.split_email(@test_email).length.should == 2
    end

    it "should the 'to' being a string" do
      @test_email.should_receive(:to){"to@example.com"}
      @send_email.split_email(@test_email).length.should == 1
    end

    it "should copy to the html_part" do
      @send_email.split_email(@test_email).first.html_part.to_s.should == @test_email.html_part.to_s
    end

    it "should handle the html_part being nil" do
      @test_email.stub(:html_part).and_return(nil)
      lambda{
        @send_email.split_email(@test_email)
      }.should_not raise_error
    end

    it "should copy the text_part" do
      @send_email.split_email(@test_email).first.text_part.to_s.should == @test_email.text_part.to_s
    end

    it "should handle the text_part being nil" do
      @test_email.stub(:text_part).and_return(nil)
      lambda{
        @send_email.split_email(@test_email)
      }.should_not raise_error
    end

    it "should include any bcc's on the first emails" do
      @test_email.bcc = 'bcc@me.com'
      @send_email.split_email(@test_email).first.bcc.should == @test_email.bcc
    end

    it "should include any cc's on the first emails" do
      @test_email.cc = 'cc@me.com'
      @send_email.split_email(@test_email).first.cc.should == @test_email.cc
    end

    it "should handle non-multipart emails" do
      @test_email=test_email{
        "This is not multipart"
      }
      @send_email.split_email(@test_email).first.body.to_s.should =~ /This is not multipart/
    end

    it "should include a cache_id in the smtp headers" do
      @email = @send_email.split_email(@test_email).first
      field=@email.header['X-SMTPAPI']
      cache_key = JSON.parse(field.value)['unique_args']['cache_key']
      cache_key.should match(/^\w{8}/)
    end

    it "should create an email for each 'to'" do
      emails=@send_email.split_email(@test_email)
      emails.each do |email|
        email.to.length.should == 1
      end
      (%w{someother@example.com to@example.com}-emails.map{|email| email.to.first}).should == []
    end

    it "should create only a single email if the same 'to' is included more than once" do
      @send_email.split_email(test_email('to@example.com')).length.should == 1
    end

  end

  def test_config
    @iron_mq_config = JSON.parse(File.read('spec/config/iron_mq.json'))['iron_mq']
    @iron_cache_config = JSON.parse(File.read('spec/config/iron_cache.json'))
    {'iron_mq'=>@iron_mq_config, 'iron_cache'=>@iron_cache_config.merge('cache_name'=>"test_email_cache"), 'delivery_method'=>'test', 'debug'=>true}
  end

  def test_email(other_tos=[])
    other_tos = [other_tos] unless other_tos.is_a?(Array)

    to = (['to@example.com']+other_tos)

    mail=Mail.new do
      to to
      from "from@example.com"
      subject "This is a test email"
      if block_given?
        body yield
      else
        text_part do
          body "This is a test"
        end
        html_part do
          content_type  'text/html; charset=UTF-8'
          body "<h1>This is a test</h1>"
        end
      end
    end
  end
    
end